import json

from html import escape as e
from typing import Dict
from inspect import isclass
from urllib.parse import urlparse
from .image import url2mime


__version__ = '0.8.1'

renderers: Dict = {}


class BaseNode:
    """
    Base Node class with reference implemention for common cases.
    This should be used as base class for all leaf level nodes
    which do not contain other nodes.
    """
    type = "prose-mirror_content-type"
    wrap_tag: str = ""
    css_class: str = ""

    def is_renderable(self, node):
        """
        Checks whether the node is worth rendering.
        Example: Image block without src attribute might not.
        """
        return True

    def render(self, in_data) -> str:
        out = ''
        if self.is_renderable(in_data):
            out = self.inner_render(in_data)
            css_class_s = f' class="{self.css_class}"' if self.css_class else ''
            if self.wrap_tag:
                out = f"<{self.wrap_tag}{css_class_s}>{out}</{self.wrap_tag}>"
        return out

    def inner_render(self, node) -> str:
        return e(node["content"]["text"])


class BaseContainer(BaseNode):
    """
    Base class for container type nodes which may further contain other nodes.
    """

    def inner_render(self, nodes: Dict) -> str:
        out = ""
        for node in nodes.get("content", []):
            node_type = node.get("type")
            renderer = renderers.get(node_type)
            assert renderer, f'Unsupported node_type: "{node_type}"'
            if renderer:
                out += renderer.render(node)
        return out


class config:
    """
    Config class to store constans which are used by the othe nodes.
    """
    DOMAIN = "python.org"


class Text(BaseNode):
    type = "text"
    mark_tags = {"bold": "strong", "italic": "em", "link": "a", "sup": "sup"}

    def inner_render(self, node):
        text = e(node["text"])
        marks = node.get("marks")
        if marks:
            for mark in marks:
                tag = self.mark_tags.get(mark.get("type"))
                attrs = mark.get("attrs")
                if attrs:
                    if tag == "a":
                        url = attrs.get("href") or ""
                        if not is_trusted_link(url):
                            attrs["target"] = "_blank"
                            attrs["rel"] = "noopener nofollow"
                    attrs_s = " ".join(
                        f'{k}="{e(v)}"' for k, v in attrs.items()
                    )
                    text = f"<{tag} {attrs_s}>{text}</{tag}>"
                else:
                    text = f"<{tag}>{text}</{tag}>"
        return text


class Heading(BaseContainer):
    type = "heading"

    def inner_render(self, node) -> str:
        attrs = node['attrs']
        level = attrs.get('level') or 1
        tag = e(f"h{level}")
        inner_html = super().inner_render(node)
        return f"<{tag}>{inner_html}</{tag}>"


class Image(BaseNode):
    type = "image"
    wrap_tag: str = "figure"

    def is_renderable(self, node):
        attrs = node.get("attrs", {})
        src = attrs.get("src", "")
        return src and bool(
            src.get('image', '').strip() or src.get('fallback', '').strip()
        )

    def inner_render(self, node) -> str:
        attrs = node["attrs"]
        alt = attrs.get('alt', '').strip()
        caption = attrs.get('caption', '').strip()
        image_url = attrs['src']['image']
        fallback_url = attrs['src']['fallback']
        image_type = url2mime(image_url)
        fallback_type = url2mime(fallback_url)
        image_src = f'<img src="{fallback_url}"/>'
        if alt:
            image_src = f'<img src="{fallback_url}" alt="{e(alt)}"/>'
        html = f'<picture><source srcset="{image_url}" type="{image_type}"/><source srcset="{fallback_url}" type="{fallback_type}"/>{image_src}</picture>'  # noqa: E501
        if caption:
            html = html + f'<figcaption>{e(caption)}</figcaption>'
        return html


class Embed(BaseContainer):
    type = "embed"

    def inner_render(self, node) -> str:
        attrs = node['attrs']
        html = attrs.get('html', '')
        if attrs.get('type') == 'video':
            caption = (attrs.get('caption') or '').strip()
            if caption:
                html += f"<figcaption>{caption}</figcaption>"
        provider_name = attrs.get('provider') or 'link'
        return f'<div class="embed-wrapper {provider_name.lower()}-wrapper"><figure>{html}</figure></div>'  # noqa: E501


class Title(BaseContainer):
    type = "title"
    wrap_tag: str = "h1"


class Paragraph(BaseContainer):
    type = "paragraph"
    wrap_tag: str = "p"

    def is_renderable(self, node):
        return bool(node.get('content'))


class BlockQuote(BaseContainer):
    type = "blockquote"
    wrap_tag: str = "blockquote"


class HardBreak(BaseContainer):
    type = "hard_break"

    def inner_render(self, node):
        return "<br>"


class HorizontalRule(BaseNode):
    type = "horizontal_rule"

    def inner_render(self, node):
        return "<hr>"


class ListItem(BaseContainer):
    type = "list_item"
    wrap_tag: str = "li"


class BulletList(BaseContainer):
    type = "bullet_list"
    wrap_tag: str = "ul"


class Doc(BaseContainer):
    type = "doc"


class OrderedList(BaseContainer):
    type = "ordered_list"
    wrap_tag: str = "ol"


def register_renderer(cls):
    renderers[cls.type] = cls()


for o in tuple(locals().values()):
    if isclass(o) and issubclass(o, BaseNode):
        register_renderer(o)


def convert_any(in_data):
    typ = in_data.get("type")
    renderer = renderers.get(typ)
    return renderer.render(in_data)


def is_trusted_link(url):
    """
    Check if the domain is the same as in the config.
    """
    link = urlparse(url)
    # Getting the domain of the link
    link = link.netloc
    return link.endswith(config.DOMAIN)


def to_html(s):
    in_data = s if isinstance(s, dict) else json.loads(s)
    return convert_any(in_data)


if __name__ == "__main__":
    import timeit

    s = open("tests/data.json").read()
    print(to_html(s))
    print(
        timeit.timeit(
            "to_html(s)",
            setup="from __main__ import to_html, s", number=100000
        )
    )



import os
import pytest
import tiptapy
from tiptapy import extras


tags_to_test = (
    "simple",
    "blockquote",
    "bulletlist",
    "mark_tags",
    "ordered_list",
    "paragraph",
    "paragraph-is_renderable",
    "image",
    "image-missing_caption",
    "image-no_caption",
    "image-mime_type",
    "featuredimage",
    "featuredimage-missing_caption",
    "featuredimage-no_caption",
    "featuredimage-mime_type",
    "horizontal_rule",
    "embed",
    "embed-missing_caption",
    "embed-no_caption",
    "heading",
    "is_renderable",
    "audio",
    "audio-no_caption",
)


def build_test_data():
    """
    Scan data directories and return test data
    """
    store = {'json': {}, 'html': {}}
    for data_type in store:
        dir_path = os.path.abspath(f'tests/data/{data_type}/')
        for file in os.listdir(dir_path):
            file_path = os.path.join(dir_path, file)
            with open(file_path) as f:
                data = f.read()
                store[data_type][file.split(f'.{data_type}')[0]] = data
    return store['json'], store['html']


json_data, html_data = build_test_data()


@pytest.mark.parametrize("tag", tags_to_test)
def test_html_tag(tag):
    """
    Test expected json input with the expected html.
    """
    tag_data = json_data[tag]
    expected_html = html_data[tag]
    assert tiptapy.to_html(tag_data) == expected_html.strip()



import pkgutil
from . import BaseNode, Image, register_renderer, e


class FeaturedImage(Image):
    type = "featuredimage"
    css_class = "featured-image"


class StackAudio(BaseNode):
    type = "audio"

    def is_renderable(self, node):
        attrs = node.get("attrs", {})
        return bool(attrs.get("src", ""))

    def inner_render(self, node):
        attrs = node["attrs"]
        src = attrs.get('src', '').strip()
        caption = attrs.get('caption', '').strip()
        audio_src_block = f'<audio src={src}></audio>'
        audio_player_block = pkgutil.get_data(
            __name__, 'templates/stack-audio-player.html'
        ).decode()
        html = f'<div>{audio_src_block}{audio_player_block}</div>'
        if caption:
            html = html + f'<figcaption>{e(caption)}</figcaption>'
        return f'<figure class="audio-player-container">{html}</figure>'


register_renderer(FeaturedImage)
register_renderer(StackAudio)



import tiptapy.image


def test_mime():
    assert tiptapy.image.SUPPORTED_FORMATS_MAP['JPEG'] == 'image/jpeg'
    assert tiptapy.image.SUPPORTED_FORMATS_MAP['JPG'] == 'image/jpeg'
    assert tiptapy.image.url2mime('https://example.com/coolcat.webp') == 'image/webp'




from setuptools import setup, find_packages


setup(
    name="tiptapy",
    version='0.8.1',  #TODO: why bumpversion works only for single quotes?
    url="https://github.com/scrolltech/tiptapy",
    description="Library that generates HTML output from JSON export of tiptap editor",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    author="Shekhar Tiwatne",
    author_email="pythonic@gmail.com",
    license="http://www.opensource.org/licenses/mit-license.php",
    package_data={'tiptapy': ['templates/stack-audio-player.html']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# Image file type and it's MIME type mappings that are suported by tiptapy.
# Detailed documentation can be found about Image file type and format guide.
# Link: https://developer.mozilla.org/en-US/docs/Web/Media/Formats/Image_types
from os.path import splitext

class SupportedFormatsMapper(dict):

    def __missing__(self, ext):
        return 'image'


SUPPORTED_FORMATS_MAP = SupportedFormatsMapper(
    PNG='image/png',
    JPG='image/jpeg',
    JPEG='image/jpeg',
    GIF='image/gif',
    BMP='image/bmp',
    WEBP='image/webp',
    SVG='image/svg+xml'
)


def url2mime(url):
    ext = splitext(url)[-1]
    ext = (ext[1:] if ext.startswith('.') else ext).upper()
    return SUPPORTED_FORMATS_MAP[ext]
